<?php

namespace Drupal\entity_toolbar\Toolbar;

use Drupal\Core\Url;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Menu\MenuLinkTreeInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Menu\MenuTreeParameters;
use Drupal\Core\Menu\MenuLinkManagerInterface;

/**
 * Toolbar integration handler.
 */
class EntityToolbarHandler implements ContainerInjectionInterface {

  use StringTranslationTrait;

  /**
   * The menu link tree service.
   *
   * @var \Drupal\Core\Menu\MenuLinkTreeInterface
   */
  protected $menuLinkTree;

  /**
   * The menu link manager.
   *
   * @var \Drupal\Core\Menu\MenuLinkManagerInterface
   */
  protected $menuLinkManager;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $account;

  /**
   * ToolbarHandler constructor.
   *
   * @param \Drupal\Core\Menu\MenuLinkTreeInterface $menu_link_tree
   *   The menu link tree service.
   * @param \Drupal\Core\Menu\MenuLinkManagerInterface $menu_link_manager
   *   The menu link manager.
   * @param \Drupal\Core\Session\AccountProxyInterface $account
   *   The current user.
   */
  public function __construct(MenuLinkTreeInterface $menu_link_tree, MenuLinkManagerInterface $menu_link_manager, AccountProxyInterface $account) {
    $this->menuLinkTree = $menu_link_tree;
    $this->menuLinkManager = $menu_link_manager;
    $this->account = $account;

  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('toolbar.menu_tree'),
      $container->get('plugin.manager.menu.link'),
      $container->get('current_user')
    );
  }

  /**
   * Hook bridge.
   *
   * @return array
   *   The entity toolbar items render array.
   *
   * @see hook_toolbar()
   */
  public function toolbar() {

    $items = [];

    $entity_toolbars = \Drupal::entityTypeManager()
      ->getStorage('entity_toolbar')
      ->loadMultiple();

    /** @var \Drupal\entity_toolbar\Entity\EntityToolbarConfig $toolbar */
    foreach ($entity_toolbars as $toolbar) {

      if (!$toolbar->status()) {
        continue;
      }

      $baseRouteName = $toolbar->get('baseRouteName');

      if (empty($baseRouteName)) {
        continue;
      }

      $menu_links = $this->menuLinkManager
        ->loadLinksByRoute($baseRouteName);

      $link = $menu_links[$baseRouteName];

      // Mimics the admin_toolbar classes to use the drop down menus in
      // the admin_toolbar/toolbar.tree library.
      $build = [
        '#type' => 'container',
        '#attributes' => ['class' => ['toolbar-menu-administration', 'entity-toolbar-menu']],
        '#attached' => ['library' => ['entity_toolbar/entity-toolbar']],
      ];

      $menu_parameters = new MenuTreeParameters();
      $menu_parameters->setMaxDepth(4);
      $menu_parameters->setRoot($link->getPluginId());
      $menu_parameters->excludeRoot();

      $tree = $this->menuLinkTree->load('entity-toolbar', $menu_parameters);

      $manipulators = [
        ['callable' => 'menu.default_tree_manipulators:checkAccess'],
        ['callable' => 'menu.default_tree_manipulators:generateIndexAndSort'],
      ];

      $tree = $this->menuLinkTree->transform($tree, $manipulators);

      $tree_build = $this->menuLinkTree->build($tree);

      // Add weight to items to prevent them being re-ordered
      // by key in menu--toolbar.html.twig.
      $index = 0;
      foreach ($tree_build['#items'] as $key => $value) {
        $value['#weight'] = $index;
        $tree_build['#items'][$key] = $value;
        $index++;
      }

      $bundleDefinition = \Drupal::entityTypeManager()->getDefinition($toolbar->get('bundleEntityId'));
      if ($bundleDefinition) {
        $toolbar->addCacheTags($bundleDefinition->getListCacheTags());
      }

      $tree_build['#cache']['tags'] += $toolbar->getCacheTags();

      $build[] = $tree_build;

      $items[$toolbar->id()] = [
        '#cache' => [
          'tags' => $toolbar->getCacheTags(),
          'contexts' => ['user.permissions'],
        ],
      ];

      $url = Url::fromRoute($baseRouteName);

      if ($url->access($this->account) && $this->account->hasPermission('access toolbar')) {
        $items[$toolbar->id()] += [
          '#type' => 'toolbar_item',
          '#weight' => -14,
          'tab' => [
            '#type' => 'link',
            '#title' => $toolbar->label(),
            '#url' => $url,
            '#attributes' => [
              'title' => $toolbar->label(),
              'class' => ['toolbar-icon', 'toolbar-icon-entity-toolbar'],
            ],
          ],
          'tray' => [
            '#heading' => $toolbar->label(),
            'entity_bundle_menu' => $build,
          ],
        ];
      }
    }

    return $items;
  }

}
